import {
  AnimationInformationPayload,
  DemosPayload,
  LanguageInformationPayload,
  Message,
  VariantInformationPayload,
  ViewerAPIMessage,
  ViewerAPIMessageSource,
  ViewerStatus,
} from './types';
import { Visao } from './visao';
import DoneCallback = jest.DoneCallback;
import { Demo } from './demo';

const VIEWER_ID = 'viewer';
const LANGUAGE = 'en';
const LANGUAGES = ['en', 'fr'];
const VARIANT = 'green';
const VARIANTS = ['green', 'blue'];
const ANIMATION = 'walk';
const ANIMATIONS = ['walk', 'run'];
const DEMOS = [
  new Demo({
    id: 'demo id',
    name: 'demo name',
  }),
];
const LOADED_STATUS_MESSAGE = {
  source: ViewerAPIMessageSource.VIEWER,
  type: ViewerAPIMessage.SEND_STATUS,
  payload: {
    status: ViewerStatus.LOADED,
  },
};
const LANGUAGE_INFORMATION_MESSAGE = {
  source: ViewerAPIMessageSource.VIEWER,
  type: ViewerAPIMessage.SEND_LANGUAGE_INFO,
  payload: {
    language: LANGUAGE,
    languages: LANGUAGES,
  },
};
const VARIANT_INFORMATION_MESSAGE = {
  source: ViewerAPIMessageSource.VIEWER,
  type: ViewerAPIMessage.SEND_VARIANT_INFO,
  payload: {
    modelVariant: VARIANT,
    modelVariants: VARIANTS,
  },
};
const ANIMATION_INFORMATION_MESSAGE = {
  source: ViewerAPIMessageSource.VIEWER,
  type: ViewerAPIMessage.SEND_ANIMATION_INFO,
  payload: {
    animation: ANIMATION,
    animations: ANIMATIONS,
  },
};
const DEMOS_MESSAGE = {
  source: ViewerAPIMessageSource.VIEWER,
  type: ViewerAPIMessage.SEND_DEMOS,
  payload: {
    demos: DEMOS,
  },
};

describe('testing message reception', () => {
  let viewer: Visao;

  beforeAll(() => {
    document.body.innerHTML = `<div><iframe id="${VIEWER_ID}"></iframe></div>`;
  });

  beforeEach(() => {
    viewer = new Visao(VIEWER_ID);
    viewer.setViewerElement(
      document.getElementById(VIEWER_ID) as HTMLIFrameElement,
    );
  });

  describe('when listening to viewer status', () => {
    it('should return new statuses on changes via callback', (doneCallback) => {
      function getStatus(status: ViewerStatus) {
        try {
          expect(status).toBe(LOADED_STATUS_MESSAGE.payload.status);
          doneCallback();
        } catch (e) {
          doneCallback(e);
        }
      }
      viewer.listenToViewerStatus(getStatus);
      receiveMessageFromExternalAPI(LOADED_STATUS_MESSAGE);
    });
  });

  describe('when listening for language information changes', () => {
    beforeEach((doneCallback) => {
      waitForViewerToBeLoaded(doneCallback);
    });

    it('should return the new language information via callback', (doneCallback) => {
      function getLanguageInformation(
        languageInformation: LanguageInformationPayload,
      ) {
        try {
          expect(languageInformation.language).toBe(LANGUAGE);
          expect(languageInformation.languages).toEqual(LANGUAGES);
          doneCallback();
        } catch (e) {
          doneCallback(e);
        }
      }
      viewer.getLanguageInformation(getLanguageInformation);
      receiveMessageFromExternalAPI(LANGUAGE_INFORMATION_MESSAGE);
    });
  });

  describe('when listening for variant information changes', () => {
    beforeEach((doneCallback) => {
      waitForViewerToBeLoaded(doneCallback);
    });

    it('should return the variant information via callback', (doneCallback) => {
      function getVariantInformation(
        variantInformation: VariantInformationPayload,
      ) {
        try {
          expect(variantInformation.modelVariant).toBe(VARIANT);
          expect(variantInformation.modelVariants).toEqual(VARIANTS);
          doneCallback();
        } catch (e) {
          doneCallback(e);
        }
      }
      viewer.getVariantInformation(getVariantInformation);
      receiveMessageFromExternalAPI(VARIANT_INFORMATION_MESSAGE);
    });
  });

  describe('when listening for animation information changes', () => {
    beforeEach((doneCallback) => {
      waitForViewerToBeLoaded(doneCallback);
    });

    it('should return the animation information via callback', (doneCallback) => {
      function getAnimationInformation(
        variantInformation: AnimationInformationPayload,
      ) {
        try {
          expect(variantInformation.animation).toBe(ANIMATION);
          expect(variantInformation.animations).toEqual(ANIMATIONS);
          doneCallback();
        } catch (e) {
          doneCallback(e);
        }
      }
      viewer.getAnimationInformation(getAnimationInformation);
      receiveMessageFromExternalAPI(ANIMATION_INFORMATION_MESSAGE);
    });
  });

  describe('when listening for demos', () => {
    beforeEach((doneCallback) => {
      waitForViewerToBeLoaded(doneCallback);
    });

    it('should return the demos via callback', (doneCallback) => {
      function getDemos(demosPayload: DemosPayload) {
        try {
          expect(demosPayload.demos).toEqual(DEMOS);
          doneCallback();
        } catch (e) {
          doneCallback(e);
        }
      }
      viewer.getDemos(getDemos);
      receiveMessageFromExternalAPI(DEMOS_MESSAGE);
    });
  });

  function receiveMessageFromExternalAPI(message: Message) {
    window.postMessage(JSON.stringify(message), '*');
  }

  function waitForViewerToBeLoaded(doneCallback: DoneCallback) {
    function getStatus() {
      try {
        doneCallback();
      } catch (e) {
        doneCallback(e);
      }
    }
    viewer.listenToViewerStatus(getStatus);
    receiveMessageFromExternalAPI(LOADED_STATUS_MESSAGE);
  }
});
