import { GetDemosCallback, GetDimensionsUnitCallback, GetLanguageInformationCallback, GetScreenshotCallback, GetVariantInformationCallback, DimensionsUnit, ViewerElement, ViewerStatusListenerCallback, ScreenshotOptions, GetAnimationInformationCallback, AnimationOptions } from './types';
/**
 * The Visao class is the starting point of the Visao viewer api.
 * It encapsulates all the necessary calls to interact with your viewers from your
 * own code base.
 */
export declare class Visao {
    private id;
    private viewerElementInternal;
    private status;
    private readonly callbacks;
    private get viewerElement();
    /**
     * #### Description
     * The constructor requires an element id given to the iframe tag/component
     * defined in your application. Internally, we take care of finding the element
     * in the DOM once the component is mounted.
     *
     * #### Usage
     *
     * ```typescript
     * const instance = new Visao("viewer-iframe-id");
     * ```
     *
     * @param id
     * The id of the viewer element.
     */
    constructor(id: string);
    /**
     * #### Description
     * > Manually set the reference of the viewer from the id.
     *
     * #### Usage:
     *
     * ```typescript
     * visao.setViewerElementFromId('viewer-id');
     * ```
     *
     *  @param id
     *  HTML element id representing the viewer
     */
    setViewerElementFromId(id: string): void;
    /**
     * #### Description
     * > Manually set the reference of the viewer element.
     *
     * #### Usage:
     *
     * ```typescript
     * const element = document.getElementById('viewer-id');
     * visao.setViewerElement(element);
     * ```
     *
     *  @param viewerElement
     *  HTML element representing the viewer
     */
    setViewerElement(viewerElement: ViewerElement): void;
    /**
     * #### Description
     * > Listens to the state of the viewer. It will notify by calling
     * the provided parameter callback providing the new state everytime
     * the state changes.
     *
     * #### Usage
     *
     * ```typescript
     * visao.listenToViewerStatus((newViewerStatus: ViewerStatus) => {
     *   // Do Something
     * });
     * ```
     *
     * @param callback
     * Callback to execute when a change happen
     */
    listenToViewerStatus(callback: ViewerStatusListenerCallback): void;
    /**
     * #### Description
     * > Removes a viewer status listener.
     *
     * #### Usage
     *
     * ```typescript
     * visao.unListenToViewerStatus(callback);
     * ```
     *
     * @param callback
     * Callback which matches the one provided to a previously called {@link listenToViewerStatus}
     */
    unListenToViewerStatus(callback: ViewerStatusListenerCallback): void;
    /**
     * #### Description
     * > Listens to any change to variants in the viewer. It will notify by calling
     * the provided parameter callback providing the new state everytime
     * the state changes.
     *
     * #### Usage
     *
     * ```typescript
     * visao.listenToVariantInfoChange((variantInformation: VariantInformationPayload) => {
     *   // Do Something
     * });
     * ```
     *
     * @param callback
     * Callback to execute when a change happen
     */
    listenToVariantInfoChange(callback: GetVariantInformationCallback): void;
    /**
     * #### Description
     * > Removes a variant info change listener.
     *
     * #### Usage
     *
     * ```typescript
     * visao.unListenToVariantInfoChange(callback);
     * ```
     *
     * @param callback
     * Callback which matches the one provided to a previously called {@link listenToVariantInfoChange}
     */
    unListenToVariantInfoChange(callback: GetVariantInformationCallback): void;
    /**
     * #### Description
     * > Listens to any change to animations in the viewer. It will notify by calling
     * the provided parameter callback providing the new state everytime
     * the state changes.
     *
     * #### Usage
     *
     * ```typescript
     * visao.listenToAnimationInfoChange((animationInformation: AnimationInformationPayload) => {
     *   // Do Something
     * });
     * ```
     *
     * @param callback
     * Callback to execute when a change happen
     */
    listenToAnimationInfoChange(callback: GetAnimationInformationCallback): void;
    /**
     * #### Description
     * > Removes an animation info change listener.
     *
     * #### Usage
     *
     * ```typescript
     * visao.unListenToAnimationInfoChange(callback);
     * ```
     *
     * @param callback
     * Callback which matches the one provided to a previously called {@link listenToAnimationInfoChange}
     */
    unListenToAnimationInfoChange(callback: GetAnimationInformationCallback): void;
    /**
     * #### Description
     * > Listens to any change to the language in the viewer. It will notify by calling
     * the provided parameter callback providing the new state everytime
     * the state changes.
     *
     * #### Usage
     *
     * ```typescript
     * visao.listenToLanguageInfoChange((languageInformation: LanguageInformationPayload) => {
     *   // Do Something
     * });
     * ```
     *
     * @param callback
     * Callback to execute when a change happen
     */
    listenToLanguageInfoChange(callback: GetLanguageInformationCallback): void;
    /**
     * #### Description
     * > Removes a language info change listener.
     *
     * #### Usage
     *
     * ```typescript
     * visao.unListenToLanguageInfoChange(callback);
     * ```
     *
     * @param callback
     * Callback which matches the one provided to a previously called {@link listenToLanguageInfoChange}
     */
    unListenToLanguageInfoChange(callback: GetLanguageInformationCallback): void;
    /**
     * #### Description
     * > Listens to any change to dimensions unit in the viewer. It will notify by calling
     * the provided parameter callback providing the new state everytime
     * the state changes.
     *
     * #### Usage
     *
     * ```typescript
     * visao.listenToDimensionsUnitChange((dimensionsUnitPayload: DimensionsUnitPayload) => {
     *   // Do Something
     * });
     * ```
     *
     * @param callback
     * Callback to execute when a change happen
     */
    listenToDimensionsUnitChange(callback: GetDimensionsUnitCallback): void;
    /**
     * #### Description
     * > Removes a dimensions unit change listener.
     *
     * #### Usage
     *
     * ```typescript
     * visao.unListenToDimensionsUnitChange(callback);
     * ```
     *
     * @param callback
     * Callback which matches the one provided to a previously called {@link listenToDimensionsUnitChange}
     */
    unListenToDimensionsUnitChange(callback: GetDimensionsUnitCallback): void;
    /**
     * #### Description
     * > Listens to any change to the demos in the viewer. It will notify by calling
     * the provided parameter callback providing the new state everytime
     * the state changes.
     *
     * #### Usage
     *
     * ```typescript
     * visao.listenToDemosChange((demos: DemosPayload) => {
     *   // Do Something
     * });
     * ```
     *
     * @param callback
     * Callback to execute when a change happen
     */
    listenToDemosChange(callback: GetDemosCallback): void;
    /**
     * #### Description
     * > Removes a demos change listener.
     *
     * #### Usage
     *
     * ```typescript
     * visao.unListenToDemosChange(callback);
     * ```
     *
     * @param callback
     * Callback which matches the one provided to a previously called {@link listenToDemosChange}
     */
    unListenToDemosChange(callback: GetDemosCallback): void;
    /**
     * #### Description
     * >  Get the data for all the demos and their steps.
     *
     * #### Usage
     *  > :warning: **{@link ViewerStatus}**  must be at **{@link ViewerStatus.LOADED}** to execute.
     *
     * ```typescript
     * // Get the demos data
     * let demos;
     * visao.getDemos((payload) => {
     *   demos = payload.demos;
     *   // Do something with the data
     * });
     * ```
     * @param callback
     * Callback with a payload as parameter which includes the list of demos.
     */
    getDemos(callback: GetDemosCallback): void;
    /**
     * #### Description
     * > Displays the provided demo step.
     *
     * #### Usage
     *  > :warning: **{@link ViewerStatus}**  must be at **{@link ViewerStatus.LOADED}** to execute.
     *
     * ```typescript
     * const stepName = 'shoe laces';
     * visao.showStep(stepName);
     * ```
     *
     * @param step
     * The name or the id of the step
     */
    showStep(step: string): void;
    /**
     * #### Description
     * > Closes currently displayed demo step.
     *
     * #### Usage
     *  > :warning: **{@link ViewerStatus}**  must be at **{@link ViewerStatus.LOADED}** to execute.
     *
     * ```typescript
     * visao.closeStep();
     * ```
     */
    closeStep(): void;
    /**
     * #### Description
     * > Displays the previous demo step.
     *
     * #### Usage
     *  > :warning: **{@link ViewerStatus}**  must be at **{@link ViewerStatus.LOADED}** to execute.
     *
     * ```typescript
     * visao.previousStep();
     * ```
     */
    previousStep(): void;
    /**
     * #### Description
     * > Displays the next demo step.
     *
     * #### Usage
     *  > :warning: **{@link ViewerStatus}**  must be at **{@link ViewerStatus.LOADED}** to execute.
     *
     * ```typescript
     * visao.nextStep();
     * ```
     */
    nextStep(): void;
    /**
     * #### Description
     * > Starts the demo steps player.
     *
     * #### Usage
     * > :warning: **{@link ViewerStatus}**  must be at **{@link ViewerStatus.LOADED}** to execute.
     *
     * ```typescript
     * visao.play();
     * ```
     */
    play(): void;
    /**
     * #### Description
     * > Pauses the demo steps player.
     *
     * #### Usage
     * > :warning: **{@link ViewerStatus}**  must be at **{@link ViewerStatus.LOADED}** to execute.
     *
     * ```typescript
     * visao.pause();
     * ```
     */
    pause(): void;
    /**
     * #### Description
     * > Gets the current language information
     *
     * #### Usage
     * > :warning: **{@link ViewerStatus}**  must be either at **{@link ViewerStatus.MOUNTED}**,
     *  **{@link ViewerStatus.LOADING}**, **{@link ViewerStatus.LOADED}** to execute.
     *
     * ```typescript
     * visao.getLanguageInformation((info) => {
     *   const currentLanguage = info.language;
     *   const availableLanguages = info.languages;
     * });
     * ```
     *
     *  @param callback
     *  The callback will contain the currently displayed languages as well as all the supported languages in the viewer.
     */
    getLanguageInformation(callback: GetLanguageInformationCallback): void;
    /**
     * #### Description
     * > Tells the viewer to change the language of the different localized content if it can be found.
     * This will be visible on UI components like tooltips, menus, flyers, etc...
     *
     * #### Usage
     * > :warning: **{@link ViewerStatus}**  must be either at **{@link ViewerStatus.MOUNTED}**,
     *  **{@link ViewerStatus.LOADING}**, **{@link ViewerStatus.LOADED}** to execute.
     *
     * ```typescript
     * // Change the viewer language to french
     * visao.changeLanguage('fr');
     * ```
     *
     *  @param language
     *  The language abbreviation to apply. An exhaustive list of all valid languages
     *  can be found **[here](https://github.com/annexare/Countries/blob/master/data/languages.json)**.
     */
    changeLanguage(language: string): void;
    /**
     * #### Description
     * > Display one or more configuration/variant(s)
     * of the current 3D model if they can be found.
     *
     * #### Usage:
     * A 3D model of a shoe may support multiple independent variants,
     * such as color options for the shoe body, laces, and sole.
     * You can apply a single variant (e.g., just the body color),
     * or multiple variants at once (e.g., body, laces, and sole).
     *
     * > :warning: **{@link ViewerStatus}**  must be at **{@link ViewerStatus.LOADED}** to execute.
     *
     * ```typescript
     * // Change only the shoe body color to green
     * visao.showModelVariant('green');
     *
     * // Apply multiple variants: green body, white laces, and black sole
     * visao.showModelVariant(['green', 'white_laces', 'black_sole']);
     * ```
     *
     *  @param modelVariants
     *  A unique key or array of keys representing one or more variants that can be matched in the current 3D model schema.
     */
    showModelVariant(modelVariants: string | string[]): void;
    /**
     * #### Description
     * > Remove one or more applied configuration/variant(s)
     * from the current 3D model if they are currently active.
     *
     * #### Usage:
     * A 3D model of a shoe may have multiple variants applied simultaneously,
     * such as different colors for the body, laces, and sole.
     * You can remove a single variant, or several at once.
     *
     * > :warning: **{@link ViewerStatus}**  must be at **{@link ViewerStatus.LOADED}** to execute.
     *
     * ```typescript
     * // Remove the green body color variant
     * visao.removeModelVariant('green');
     *
     * // Remove multiple variants: white laces and black sole
     * visao.removeModelVariant(['white_laces', 'black_sole']);
     * ```
     *
     *  @param modelVariants
     *  A unique key that can be matched inside the current 3D model schema.
     */
    removeModelVariant(modelVariants: string | string[]): void;
    /**
     * #### Description
     * > Get the 3D model's variants/configurations information from the Visao viewer.
     * A configuration/variant is a structure with selected properties within the 3D model file,
     * which will determine how the 3D model will be displayed.
     *
     * #### Usage:
     * > :warning: **{@link ViewerStatus}**  must be at **{@link ViewerStatus.LOADED}** to execute.
     *
     * A 3D model of a shoe may have multiple variants applied simultaneously,
     * such as different colors for the body, laces, and sole.
     *
     * ```typescript
     * // Get the configurations/variants information
     * let variantsDisplayed;
     * let availableVariants;
     * visao.getVariantInformation((info) => {
     *   variantsDisplayed = info.selectedModelVariants;
     *   availableVariants = info.availableModelVariants;
     *   // Do something with the data
     * });
     * ```
     *
     *  @param callback
     *  The configuration/variant information will be the parameter of the provided callback once called.
     */
    getVariantInformation(callback: GetVariantInformationCallback): void;
    /**
     * #### Description
     * > Get the 3D model's animation information from the Visao viewer.
     *
     * #### Usage:
     * > :warning: **{@link ViewerStatus}**  must be at **{@link ViewerStatus.LOADED}** to execute.
     *
     * ```typescript
     * // Get the animation information
     * let availableAnimations;
     * let currentAnimation;
     * visao.getAnimationInformation((info) => {
     *   availableAnimations = info.animations;
     *   currentAnimation = info.animation;
     *   // Do something with the data
     * });
     * ```
     *
     *  @param callback
     *  The animation information will be the parameter of the provided callback once called.
     */
    getAnimationInformation(callback: GetAnimationInformationCallback): void;
    /**
     * #### Description
     * > Start the passed animation.
     *
     * #### Usage:
     * Default or step animations will be overridden by the new animation.
     *
     * > :warning: **{@link ViewerStatus}**  must be at **{@link ViewerStatus.LOADED}** to execute.
     *
     * ```typescript
     * // Start the animation
     * visao.startAnimation(animation);
     * ```
     * ```typescript
     * // Start the animation with options
     * visaoAPI.startAnimation(anim, {
     *         repetitionType: AnimationRepetitionType.REPEAT,
     *         repetitionQuantity: 2,
     *       })
     * ```
     *
     *  @param animation
     *  The animation name to start
     *
     *
     *  @param options
     *  The animation options
     */
    startAnimation(animation: string, options?: AnimationOptions): void;
    /**
     * #### Description
     * > Stop the current animation.
     *
     * #### Usage:
     * Viewer preset animations (default or step) will be restored. This can be prevented by parameter preventPresets being set to true.
     *
     * > :warning: **{@link ViewerStatus}**  must be at **{@link ViewerStatus.LOADED}** to execute.
     *
     * ```typescript
     * // Stop the animation (preset animation will be restored if available)
     * visao.stopAnimation();
     * ```
     *
     * ```typescript
     * // Stop the animation (preset animation will not be restored)
     * visao.stopAnimation(true);
     * ```
     *
     *  @param preventPresets
     *  Whether preset (default & step) animations should be prevented from being restored, Default: false
     */
    stopAnimation(preventPresets?: boolean): void;
    /**
     * #### Description
     * > Sets the dimensions unit.
     *
     * #### Usage
     *  > :warning: **{@link ViewerStatus}**  must be at **{@link ViewerStatus.LOADED}** to execute.
     *
     * ```typescript
     * const unit = 'm';
     * visao.setDimensionsUnit(unit);
     * ```
     *
     * @param unit
     * The new unit.
     */
    setDimensionsUnit(unit: DimensionsUnit): void;
    /**
     * #### Description
     * > Starts the augmented reality feature.
     *
     * #### Usage
     * > :warning: **{@link ViewerStatus}**  must be at **{@link ViewerStatus.LOADED}** to execute. This feature only
     *  works on devices supporting augmented reality.
     *
     * ```typescript
     * visao.startAR();
     * ```
     */
    startAR(): void;
    /**
     * #### Description
     * > Reset the viewer's camera to its original state.
     *
     * #### Usage
     *  > :warning: **{@link ViewerStatus}**  must be at **{@link ViewerStatus.LOADED}** to execute.
     *
     * ```typescript
     * visao.resetCamera();
     * ```
     */
    resetCamera(): void;
    /**
     * #### Description
     * > Captures a screenshot of the viewer's 3D model.
     * This will trigger a download of the image.
     *
     * #### Usage
     *  > :warning: **{@link ViewerStatus}**  must be at **{@link ViewerStatus.LOADED}** to execute.
     *
     * ```typescript
     * visao.captureScreenshot({
     *  mimeType: 'image/png',
     * });
     * ```
     * @param options
     * {@link ScreenshotOptions}
     * Optional screenshot options which can include the `mimeType` and `fileFormat`.
     * The default values are `mimeType: 'image/png'` and `fileFormat: 'blob'`.
     */
    captureScreenshot(options?: Partial<ScreenshotOptions>): void;
    /**
     * #### Description
     * >  Captures a screenshot of the viewer's 3D model and then returns the image blob.
     *
     * #### Usage
     *  > :warning: **{@link ViewerStatus}**  must be at **{@link ViewerStatus.LOADED}** to execute.
     *
     * ```typescript
     * // Get the screenshot blob
     * visao.getScreenshot((screenshot: Blob | string) => {
     *   // Do something with the screenshot
     * }, {
     *   mimeType: 'image/png',
     *   fileFormat: 'blob',
     * });
     * ```
     * @param callback
     * Callback which includes the captured screenshot blob as parameter.
     * @param options
     * {@link ScreenshotOptions}
     * Optional screenshot options which can include the `mimeType` and `fileFormat`.
     * The default values are `mimeType: 'image/png'` and `fileFormat: 'blob'`.
     */
    getScreenshot(callback: GetScreenshotCallback, options?: Partial<ScreenshotOptions>): void;
    /**
     * #### Description
     * > Locks the viewer's camera controls.
     *
     * #### Usage
     *  > :warning: **{@link ViewerStatus}**  must be at **{@link ViewerStatus.LOADED}** to execute.
     *
     * ```typescript
     * visao.lockCamera();
     * ```
     */
    lockCamera(): void;
    /**
     * #### Description
     * > Unlocks the viewer's camera controls.
     *
     * #### Usage
     *  > :warning: **{@link ViewerStatus}**  must be either at **{@link ViewerStatus.MOUNTED}**,
     *  **{@link ViewerStatus.LOADING}**, **{@link ViewerStatus.LOADED}** to execute.
     *
     * ```typescript
     * visao.unlockCamera();
     * ```
     */
    unlockCamera(): void;
    /**
     * #### Description
     * > Shows the viewer's help panel.
     *
     * #### Usage
     *  > :warning: **{@link ViewerStatus}**  must be either at **{@link ViewerStatus.MOUNTED}**,
     *  **{@link ViewerStatus.LOADING}**, **{@link ViewerStatus.LOADED}** to execute.
     *
     * ```typescript
     * visao.showHelp();
     * ```
     */
    showHelp(): void;
    /**
     * #### Description
     * > Closes the viewer's help panel.
     *
     * #### Usage
     *  > :warning: **{@link ViewerStatus}**  must be either at **{@link ViewerStatus.MOUNTED}**,
     *  **{@link ViewerStatus.LOADING}**, **{@link ViewerStatus.LOADED}** to execute.
     *
     * ```typescript
     * visao.closeHelp();
     * ```
     */
    closeHelp(): void;
    /**
     * #### Description
     * > Shows the viewer's QR code panel.
     *
     * #### Usage
     *  > :warning: **{@link ViewerStatus}**  must be either at **{@link ViewerStatus.MOUNTED}**,
     *  **{@link ViewerStatus.LOADING}**, **{@link ViewerStatus.LOADED}** to execute.
     *
     * ```typescript
     * visao.showQR();
     * ```
     */
    showQR(): void;
    /**
     * #### Description
     * > Closes the viewer's QR code panel.
     *
     * #### Usage
     *  > :warning: **{@link ViewerStatus}**  must be either at **{@link ViewerStatus.MOUNTED}**,
     *  **{@link ViewerStatus.LOADING}**, **{@link ViewerStatus.LOADED}** to execute.
     *
     * ```typescript
     * visao.closeQR();
     * ```
     */
    closeQR(): void;
    /**
     * #### Description
     * > Shows the viewer's dimensions.
     *
     * #### Usage
     * > :warning: **{@link ViewerStatus}**  must be at **{@link ViewerStatus.LOADED}** to execute.
     *
     * ```typescript
     * visao.showDimensions();
     * ```
     */
    showDimensions(): void;
    /**
     * #### Description
     * > Closes the viewer's dimensions.
     *
     * #### Usage
     * > :warning: **{@link ViewerStatus}**  must be at **{@link ViewerStatus.LOADED}** to execute.
     *
     * ```typescript
     * visao.closeDimensions();
     * ```
     */
    closeDimensions(): void;
    /**
     * #### Description
     * > Starts the viewer's auto-rotation.
     *
     * #### Usage
     * > :warning: **{@link ViewerStatus}**  must be at **{@link ViewerStatus.LOADED}** to execute.
     *
     * ```typescript
     * visao.startAutoRotate();
     * ```
     */
    startAutoRotate(): void;
    /**
     * #### Description
     * > Stops the viewer's auto-rotation.
     *
     * #### Usage
     * > :warning: **{@link ViewerStatus}**  must be at **{@link ViewerStatus.LOADED}** to execute.
     *
     * ```typescript
     * visao.stopAutoRotate();
     * ```
     */
    stopAutoRotate(): void;
    /**
     * #### Description
     * > Mutes the viewer's voiceover.
     *
     * #### Usage
     * > :warning: **{@link ViewerStatus}**  must be at **{@link ViewerStatus.MOUNTED}** to execute.
     *
     * ```typescript
     * visao.muteVoiceover();
     * ```
     */
    muteVoiceover(): void;
    /**
     * #### Description
     * > Un-mutes the viewer's voiceover.
     *
     * #### Usage
     * > :warning: **{@link ViewerStatus}**  must be at **{@link ViewerStatus.MOUNTED}** to execute.
     *
     * ```typescript
     * visao.unmuteVoiceover();
     * ```
     */
    unmuteVoiceover(): void;
    private handleIncomingMessage;
    private executeAction;
    private logInvalidViewerElement;
    private logForInsufficientStatusLevel;
    private validateStatusHasReachedNeededLevel;
}
//# sourceMappingURL=visao.d.ts.map